@@ -58,4 +58,9 @@
 #define _LARGEFILE_SOURCE
 #define _FILE_OFFSET_BITS 64
 
+#ifdef __linux__
+/* features.h uses the defines above to set feature specific defines.  */
+#include <features.h>
+#endif
+
 #endif
