@@ -16,9 +16,10 @@ start_server {tags {"tracking"}} {
 
     test {The other connection is able to get invalidations} {
         r SET a 1
+        r SET b 1
         r GET a
-        r INCR a
-        r INCR b ; # This key should not be notified, since it wasn't fetched.
+        r INCR b ; # This key should not be notified, since it wasn't fetched. 
+        r INCR a 
         set keys [lindex [$rd1 read] 2]
         assert {[llength $keys] == 1}
         assert {[lindex $keys 0] eq {a}}
@@ -107,6 +108,22 @@ start_server {tags {"tracking"}} {
         assert {$keys eq {mykey}}
     }
 
+    test {Tracking gets notification of lazy expired keys} {
+        r CLIENT TRACKING off
+        r CLIENT TRACKING on BCAST REDIRECT $redir NOLOOP
+        # Use multi-exec to expose a race where the key gets an two invalidations
+        # in the same event loop, once by the client so filtered by NOLOOP, and
+        # the second one by the lazy expire
+        r MULTI
+        r SET mykey{t} myval px 1
+        r SET mykeyotherkey{t} myval ; # We should not get it
+        r DEBUG SLEEP 0.1
+        r GET mykey{t}
+        r EXEC
+        set keys [lsort [lindex [$rd1 read] 2]]
+        assert {$keys eq {mykey{t}}}
+    } {}
+
     test {Tracking gets notification on tracking table key eviction} {
         r CLIENT TRACKING off
         r CLIENT TRACKING on REDIRECT $redir NOLOOP
